unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DB, DBTables, BDE, StdCtrls, ComCtrls, ExtCtrls, Registry;

const
  Section = 'MainWindow';
  RegPath = 'Software\4th february\Index Rebuilder\1.0';

type
  TMainFormIndexRebuilder = class(TForm)
    laAliasName: TLabel;
    cbAliasName: TComboBox;
    laTableName: TLabel;
    cbTableName: TComboBox;
    btnStartRebuild: TButton;
    btnCloseApp: TButton;
    tbWorkTable: TTable;
    Image1: TImage;
    Image2: TImage;
    Bevel1: TBevel;
    PB: TProgressBar;
    StaticText1: TStaticText;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbAliasNameChange(Sender: TObject);
    procedure btnStartRebuildClick(Sender: TObject);
    procedure btnCloseAppClick(Sender: TObject);
  private
    procedure ReadSettings;
    procedure WriteSettings;
    function RebuildIndexes(AliasName, TableName, ErrorStr: string): Boolean;
    procedure ShowStatus(StatusText: string);
  public
    FIniFile: TRegIniFile;
    procedure WindowsMessagesHandler(var Msg: TMsg; var Handled: Boolean);
    procedure ShowAbout;
  end;

var
  MainFormIndexRebuilder: TMainFormIndexRebuilder;

implementation

uses MsgStr;

{$R *.DFM}

const
  ItemID = 99;

procedure TMainFormIndexRebuilder.WindowsMessagesHandler(var Msg: TMsg; var Handled: Boolean);
begin
  if Msg.message = WM_SYSCOMMAND then
    if Msg.wParam = ItemID then
      ShowAbout;
end;

procedure TMainFormIndexRebuilder.ReadSettings;
begin
  with FIniFile, MainFormIndexRebuilder do
  begin
    Left := ReadInteger(Section, 'Left', Left);
    Top := ReadInteger(Section, 'Top', Top);
  end;
end;

procedure TMainFormIndexRebuilder.WriteSettings;
begin
  with FIniFile, MainFormIndexRebuilder do
  begin
    WriteInteger(Section, 'Left', Left);
    WriteInteger(Section, 'Top', Top);
  end;
end;

procedure TMainFormIndexRebuilder.ShowAbout;
begin
  MessageBox(Application.Handle, PChar(rsAboutText), PChar(rsAboutCaption),
    MB_ICONINFORMATION or MB_OK);
end;

procedure TMainFormIndexRebuilder.FormCreate(Sender: TObject);
begin
  MainFormIndexRebuilder.Caption := Application.Title;
  Application.OnMessage := WindowsMessagesHandler;

{ Appending menuitem to the Form's system menu }
  AppendMenu(GetSystemMenu(MainFormIndexRebuilder.Handle, False), MF_SEPARATOR,
    0, '');
  AppendMenu(GetSystemMenu(MainFormIndexRebuilder.Handle, False), MF_BYPOSITION,
    ItemID, PChar(rsAboutCaption));

{ Appending menuitem to the Application's system menu }
  AppendMenu(GetSystemMenu(Application.Handle, False), MF_SEPARATOR, 0, '');
  AppendMenu(GetSystemMenu(Application.Handle, False), MF_BYPOSITION, ItemID,
    PChar(rsAboutCaption));

  FIniFile := TRegIniFile.Create(RegPath);
  ReadSettings;

{ Populate the Alias drop down with the currently defined aliases }
  Screen.Cursor := crHourGlass;
  try
    Session.GetAliasNames(cbAliasName.Items);
    cbAliasName.ItemIndex := 0;
  except
    { dummy except block to guarantee we get here }
  end;
  Screen.Cursor := crDefault;

{ Get the table name for the first index in the list }
  cbAliasNameChange(cbAliasName);

  PB.Position := 0;
  StaticText1.Caption := '';
end;

procedure TMainFormIndexRebuilder.FormDestroy(Sender: TObject);
begin
  WriteSettings;
  FIniFile.Free;
end;

procedure TMainFormIndexRebuilder.cbAliasNameChange(Sender: TObject);
begin
{ Get the tables in the new index. }
  Screen.Cursor := crHourGlass;
  try
    Session.GetTableNames(cbAliasName.Items[cbAliasName.ItemIndex], '', True,
      False, cbTableName.Items);
    cbTableName.Items.Insert(0, rsAllTables);
    cbTableName.ItemIndex := 0;
  except
    // dummy except block to guarantee we get here.
  end;
  Screen.Cursor := crDefault;
end;

procedure TMainFormIndexRebuilder.btnStartRebuildClick(Sender: TObject);
var
  Counter: Integer;
  ErrorStr: string;
begin
  ErrorStr := '';
  if (cbTableName.ItemIndex > 0) then
  begin
    if not RebuildIndexes(cbAliasName.Items[cbAliasName.ItemIndex],
      cbTableName.Items[cbTableName.ItemIndex], ErrorStr) then
      MessageDlg(Format(rsUnableToRebuild,
        [cbTableName.Items[cbTableName.ItemIndex], ErrorStr]), mtError,
        [mbOK], 0);
  end else
  begin
    PB.Max := cbTableName.Items.Count - 1;
    for Counter := 1 to Pred(cbTableName.Items.Count) do
    begin
      PB.Position := Counter;
      if not RebuildIndexes(cbAliasName.Items.Strings[cbAliasName.ItemIndex],
        cbTableName.Items.Strings[Counter], ErrorStr) then
      begin
        if ErrorStr <> '' then
          MessageDlg(Format(rsUnableToRebuild, [cbTableName.Items[Counter],
            ErrorStr]), mtError, [mbOK], 0);
      end;
      PB.Position := 0;
    end;
  end;
  cbAliasName.SetFocus;
end;

function TMainFormIndexRebuilder.RebuildIndexes(AliasName, TableName,
  ErrorStr: string): Boolean;
var
  bdeResult: DBIResult;
begin
  Result := False;

  if (tbWorkTable.Active) then tbWorkTable.Active := False;

  tbWorkTable.DatabaseName := AliasName;
  tbWorkTable.TableName := TableName;

  Screen.Cursor := crHourGlass;
  try
    ShowStatus(Format(rsOpenStatus, [TableName]));
    tbWorkTable.Open;
  except
  end;
  Screen.Cursor := crDefault;

  if not tbWorkTable.Active then
    ErrorStr := rsNotExclusive
  else
  begin
    ShowStatus(Format(rsBuildStatus, [TableName]));
    Screen.Cursor := crHourGlass;
    try
      bdeResult := DbiRegenIndexes(tbWorkTable.Handle);
      case bdeResult of
        DBIERR_NONE:           Result := True;
        DBIERR_INVALIDHNDL:    ErrorStr := rsInvalidHandle;
        DBIERR_NEEDEXCLACCESS: ErrorStr := rsSharedMode;
        DBIERR_NOTSUPPORTED:   ErrorStr := rsRemote;
      else
      	ErrorStr := rsUnexpected;
      end;
    except
      // dummy
    end;
    Screen.Cursor := crDefault;
  end;
  ShowStatus('');
end;

procedure TMainFormIndexRebuilder.ShowStatus(StatusText: string);
begin
  StaticText1.Caption := StatusText;
  StaticText1.Update;
end;

procedure TMainFormIndexRebuilder.btnCloseAppClick(Sender: TObject);
begin
  MainFormIndexRebuilder.Close;
end;

end.
